﻿using System;
using System.Reflection;
using System.IO;
using System.CodeDom;
using System.CodeDom.Compiler;
using Microsoft.CSharp;
using Eds.AirSoa.Util.Messaging;
using System.Diagnostics;
using System.Linq;

namespace RefactorMessages
{
    /// <summary> 
    /// This code example creates a graph using a CodeCompileUnit and   
    /// generates source code for the graph using the CSharpCodeProvider. 
    /// </summary> 
    public class GenerateCodeMessageKeyClass
    {
        public enum DirectiveType
        {
            None,
            OnlyStart,
            OnlyEnd,
            Both,
        }

        /// <summary> 
        /// Define the compile unit to use for code generation.  
        /// </summary>
        CodeCompileUnit targetUnit;

        /// <summary> 
        /// The only class in the compile unit. This class contains 2 fields, 
        /// 3 properties, a constructor, an entry point, and 1 simple method.  
        /// </summary>
        CodeTypeDeclaration targetClass;

        /// <summary> 
        /// Define the class. 
        /// </summary> 
        public GenerateCodeMessageKeyClass(Messages messages, string tower)
        {
            targetUnit = new CodeCompileUnit();

            CodeNamespace clsMessageKeys = new CodeNamespace("LoggingSupport");

            clsMessageKeys.Imports.Add(new CodeNamespaceImport("System"));
            clsMessageKeys.Imports.Add(new CodeNamespaceImport("Messaging"));

            targetClass = new CodeTypeDeclaration(tower.ToString().ToUpper() + "MessageKeys");
            targetClass.IsClass = true;
            targetClass.TypeAttributes = TypeAttributes.Public;

            clsMessageKeys.Types.Add(targetClass);
            targetUnit.Namespaces.Add(clsMessageKeys);

            foreach (var msg in messages.MessagesByTypes)
            {
                AddSubClass(msg);
            }
        }

        /// <summary> 
        /// Add an entry point to the class. 
        /// </summary> 
        public void AddSubClass(MessagesByType messagesByType)
        {
            CodeTypeDeclaration cls = new CodeTypeDeclaration();
            cls.Name = string.Join("", messagesByType.typeName.Split(' ')) + "Messages";
            cls.IsClass = true;
            cls.TypeAttributes = TypeAttributes.Public;
            cls.Attributes = MemberAttributes.Static;

            string comment;

            switch (messagesByType.typeCode)
            {
                case "1":
                    comment = "Validation Error 200~399";
                    break;

                case "2":
                    comment = "Application Error 400~799";
                    break;

                case "3":
                    comment = "System Error 800~999";
                    break;

                case "4":
                    comment = "Info Messages 001~099";
                    break;

                case "5":
                    comment = "Warn Messages 100~199";
                    break;

                default:
                    comment = string.Empty;
                    break;

            }
            cls.Comments.Add(new CodeCommentStatement(comment));

            var query = messagesByType.Messages.GroupBy(o => o.category);

            foreach (var grp in query)
            {
                int index = 0;

                var list = grp.ToList();
                foreach (var msg in list)
                {
                    DirectiveType directiveType = DirectiveType.None;

                    if (!string.IsNullOrEmpty(grp.Key))
                    {
                        if (list.Count == 1) { directiveType = DirectiveType.Both; }
                        else if (index == 0) { directiveType = DirectiveType.OnlyStart; }
                        else if (index == list.Count - 1) { directiveType = DirectiveType.OnlyEnd; }
                    }

                    AddFields(directiveType, grp.Key, cls, msg);

                    index++;
                }


            }

            targetClass.Members.Add(cls);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="flagEndOrStart">-1 start; 0 content; 1 end; 2 both for one field</param>
        /// <param name="cls"></param>
        /// <param name="msg"></param>
        public void AddFields(DirectiveType directiveType, string categoryName, CodeTypeDeclaration cls, Message msg)
        {
            // Declare the widthValue field.
            CodeMemberField field = new CodeMemberField();
            field.Attributes = MemberAttributes.Public | MemberAttributes.Static;
            field.Name = msg.ClassName;
            field.Type = new CodeTypeReference("readonly MessageKey");

            string messageType;

            switch (msg.TypeCode)
            {
                case "1":
                    messageType = "MessageType.VALIDATION_ERROR";
                    break;

                case "2":
                    messageType = "MessageType.APPLICATION_ERROR";
                    break;

                case "3":
                    messageType = "MessageType.SYSTEM_ERROR";
                    break;

                case "4":
                    messageType = "MessageType.INFO";
                    break;

                case "5":
                    messageType = "MessageType.WARN";
                    break;

                default:
                    messageType = "MessageType.UNASSIGNED";
                    Debug.Assert(false, "need to modify spreadsheet to specify the type");
                    break;
            }

            field.InitExpression = new CodeObjectCreateExpression(
                new CodeTypeReference("MessageKey"),
                new CodeTypeReferenceExpression(messageType),
                new CodePrimitiveExpression(msg.id));

            // Add region
            if (directiveType == DirectiveType.OnlyStart || directiveType == DirectiveType.Both)
            {
                field.StartDirectives.Add(new CodeRegionDirective { RegionMode = CodeRegionMode.Start, RegionText = categoryName });
            }
            if (directiveType == DirectiveType.OnlyEnd || directiveType == DirectiveType.Both)
            {
                field.EndDirectives.Add(new CodeRegionDirective { RegionMode = CodeRegionMode.End, RegionText = categoryName });
            }

            cls.Members.Add(field);
        }

        /// <summary> 
        /// Generate CSharp source code from the compile unit. 
        /// </summary> 
        /// <param name="filename">Output file name</param>
        public void Generate(string fileName)
        {
            CodeDomProvider provider = CodeDomProvider.CreateProvider("CSharp");
            provider.Supports(GeneratorSupport.PublicStaticMembers);

            CodeGeneratorOptions options = new CodeGeneratorOptions();
            options.BracingStyle = "Block";

            using (StreamWriter sourceWriter = new StreamWriter(fileName))
            {
                provider.GenerateCodeFromCompileUnit(
                    targetUnit, sourceWriter, options);
            }
        }
    }
}
